/*----------------------------------------------------------------------------	
Example program for AKKON USB Controller Baord V1.0
Author: Gerhard Burger
Date: 24.01.2006
Compiler: Microchip C18 demo version
Last update: 
Send bugs/infos to: geburger@aon.at

This program is designed for the AKKON USB Controller board

It will be shown:
1.) how to program output pins of the board (switch on Leds)
2.) how to read an input, in the example the firmware update key will be checked
3.) how to use the serial interface (19200 Baud, 8 data bits, 1 Stop, no parity)
	Cable has to be a 1:1 (no twisted pair)
4.) how to use the system timer implemented with the AKKON USB Controller board
    that will be incremented every 1ms by an interrupt

After initialisation the program sends a startup message to the serial interface.
Then running in while loop the progrm controls two blinking Leds using the system
tick timer and change state ever 500 ms.
Furtheron if the firmware update switch is pressed pin SMZ_CLK will be set to 1
and consequentyl LED on SMZ_CLK will be 1
If the user sends data over the RS232 to the AKKON USB controller board, the 
data will send back to the sender


For compiling the program, please set the position of the linker script and the
directory where to put the output bin file (EX1_AKKON.hex). It can be done by 
the menu point
Project-->Build options-->Project: 
set output dirctory e.g. to:  C:\PIC_Ex\EX1_AKKON\_output
set Linker-Script path e.g. to: C:\PIC_Ex\EX1_AKKON and the
press function key F10 to compile 

------------------------------------------------------------------------------*/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include "user\PicHardw.h"

/** V A R I A B L E S ********************************************************/
#pragma udata

/** P R I V A T E  P R O T O T Y P E S ***************************************/
// void ThisProcedure...
/** D E C L A R A T I O N S **************************************************/
#pragma code

#define myDelay 500 // 500 ms

void main(void)
{
	unsigned char StartStr[] = "\r\nAKKON USB controller board V1.0\r\n";
	unsigned char InfoStr[]	= "please press a key\r\n";
	unsigned long TimeLimit; // save last change of output stte
	unsigned char RS232_inData;

	InituPHardW();		// initialize hardware of the AKKON USB controller board
	SendString(StartStr);
	SendString(InfoStr);
	TimeLimit = SysTicks;  	// save current time
	while (1==1)  // loop forerver
	{
		if (SysTicks >= TimeLimit)		// check if time value has been reached
		{
			if (SMX_CLK == 0) SMX_CLK = 1;
						else  SMX_CLK = 0;
			SMY_CLK = ~SMX_CLK;
			
			TimeLimit = SysTicks + myDelay;  // calculate time limit for next output change
		}
		if (SW2 != 0) SMZ_CLK = 1;		// check firmwar update pin and set output pin to 1 or 0
				 else SMZ_CLK = 0;

		if (ReceiveSerial(&RS232_inData) != 0) TXbyte(RS232_inData);  // if data received then send it back

	}	
}


