#undef TG_XP
#define PIC

#include "picHardW.h"	// hardware definition
#include <p18cxxx.h>

volatile unsigned long SysTicks;  // Systemuhr 1ms

#define GetLimitSwitchState (PORTB & 0xFC);

/** V E C T O R  R E M A P P I N G *******************************************/

extern void _startup (void);        // See c018i.c in your C18 compiler dir

// Interrupt-Funktion fr das Timing der Schrittmotoransteuerung
#pragma interrupt high_interrupt_function
void high_interrupt_function(void)
{
// Timer 0 ist zustdig fr das Timing des Feed
// TMR0L und TMR0H werden immer wieder neu gesetzt
// und bei berlauf von der Interrupt-Rouinte ausgeschaletet
// TM0IF wird laufend abgefragt und bei Erreichen 
// der nchste Reloadwert gesetzt
	INTCONbits.T0IE = 0;	// switch of Timer 0 interrupts, will be retriggered by software
}


// Erzeugt alle 1 ms einen Interrupt und erhht dabei de Variable SysTicks.
#pragma interruptlow low_interrupt_function
void low_interrupt_function(void)
{
	PIE1bits.TMR1IE = 0;	   // switch of Timer 1 interrupts, will be retriggered by software
	SysTicks++;
	TMR1L=TMR1_RELOAD_1MS_LO;
	TMR1H=TMR1_RELOAD_1MS_HI;  // 65040  (delta = 495) 48.000.000/4/Pre=8/ =fcpu/4/8/495= 3Hz, gemessen 1KHz
	PIR1bits.TMR1IF = 0;	   // clear intrrupt flag
	PIE1bits.TMR1IE = 1;
}

#pragma code _RESET_INTERRUPT_VECTOR = 0x000800
void _reset (void)
{
    _asm goto _startup _endasm
}
#pragma code

#pragma code _HIGH_INTERRUPT_VECTOR = 0x000808
void _high_ISR (void)
{   
	if (INTCONbits.TMR0IF != 0)
    {
        _asm GOTO high_interrupt_function _endasm
    }
}

#pragma code _LOW_INTERRUPT_VECTOR = 0x000818
void _low_ISR (void)
{
	if (PIR1bits.TMR1IF != 0)
    {
        _asm GOTO low_interrupt_function _endasm
    }
}
#pragma code


// Timer0 Timer fr den Vorschub
// Timer1 Timer fr Timeouts
// Frequenz von Timer0 46875 Hz  => Feed [mm/s] = f / Reloadwert * Resolution
void InitFeedTimer(void)
{
#ifndef TG_XP 
	T0CONbits.T08BIT = 0;
	T0CONbits.T0CS = 0;	// internal clock fcpu /4 
	T0CONbits.PSA = 0;	// sitich on Prescaler
//  divide by 2 pow (7+1) = 46875 KHz
	T0CONbits.T0PS2 = 1;	// Prescaler by 256
	T0CONbits.T0PS1 = 1;	// Prescaler by 256
	T0CONbits.T0PS0 = 1;	// Prescaler by 256 
//	INTCON2bits.TMR0IP = 0;
    INTCONbits.TMR0IF = 0;
    INTCONbits.TMR0IE = 1;
	T0CONbits.TMR0ON = 1;
#endif

}


// Timer1 Timer fr Timeouts, resp. Systick-Variable
void InitSystemTimer(void)
{
#ifndef TG_XP
	T1CONbits.RD16 = 1;
	T1CONbits.T1RUN = 1;	// internal clock fcpu /4 
	T1CONbits.T1CKPS1 = 1;	// low-high transition
	T1CONbits.T1CKPS0 = 1;	// low-high transition
// set prescaler divide by 8   fOutput : 
	T1CONbits.T1OSCEN = 0;
	T1CONbits.T1SYNC = 0;
	T1CONbits.TMR1CS = 0; // internal clock fcpu / 4
	T1CONbits.TMR1ON = 1; // Timer on;
	TMR1L=TMR1_RELOAD_1MS_LO;
	TMR1H=TMR1_RELOAD_1MS_HI;
    PIE1bits.TMR1IE = 1;	// Timer interrupt on
    PIR1bits.TMR1IF = 0;	// Timer interrupt on
#endif
	SysTicks = 0;
}




void InitPorts(void)
{
	TRIS_SMX_CLK  = 0;  // Set as Output
	TRIS_SMY_CLK  = 0;
	TRIS_SMZ_CLK  = 0;
	TRIS_SMC_CLK  = 0;

	TRIS_SMX_DIR = 0;
	TRIS_SMY_DIR = 0;
	TRIS_SMZ_DIR = 0;
	TRIS_SMC_DIR = 0;

	TRIS_SMX_IMIN  = 0;
	TRIS_SMY_IMIN  = 0;
	TRIS_SMZ_IMIN  = 0;
	TRIS_SMC_IMIN  = 0;  // hold current

	TRIS_SM_FULL = 0;	// Full step mode pin
	TRIS_SM_ENABLE_ALL = 0;
	TRIS_M08	 = 0;
	TRIS_M10	 = 0;
	TRIS_PWM_SPINDLE = 0;

// configure limit switches
	TRIS_MIN_X = 1;
	TRIS_MIN_Y = 1;
	TRIS_MIN_Z = 1;
	TRIS_MAX_X = 1;
	TRIS_MAX_Y = 1;
	TRIS_MAX_Z = 1;

	TRIS_BTN_EMERGENCY = 1; // configure emergency button
	TRIS_HALL_SENSOR_IN = 1;

	SM_FULL = 0; 			// Full step mode
	SM_ENABLE_ALL = 0;			// 
	SMX_CLK = 0;			// 
	SMY_CLK = 0;			// 
	SMZ_CLK = 0;			// 
	SMC_CLK = 0;			// 
}


/* InituPHardW */
void InituPHardW(void)
{
#ifndef TG_XP
	INTCONbits.GIE = 0; // disable interrupts
#endif
	InitPorts();
	InitFeedTimer();
	InitSystemTimer();
	InitRS232();


#ifndef TG_XP
// Startup-Code for Interrupt Handling
	INTCONbits.GIEL = 1;
	INTCONbits.GIEH = 1;
	INTCON2bits.TMR0IP = 1;
	IPR1bits.TMR1IP = 0;
	RCONbits.IPEN=1;
	PIE1bits.TMR1IE =1;
	INTCONbits.GIE = 1; // run the timer*/
#endif
    ADCON1 |= 0x0F; // Configure firmwar upadte pine
	TRISAbits.TRISA2 = 1; // set Pin 2 of Port as input
}



//
void InitRS232(void)
{
	unsigned char temp;
	PIR1bits.TXIF = 0;
	TRIS_RX_RS232 = 1;
	TRIS_TX_RS232 = 0;	// set as output
	SSPCON1 = 1;   // Make sure SPI is disabled
	SPBRG = 0x71; 	
    SPBRGH = 0x02;      // 0x0271 for 48MHz -> 19200 baud
//	SPBRG = 69;
//	SPBRGH = 0;
		// 19200 with 20 MHz 
    TXSTA = 0x24;       // TX enable BRGH=1
    RCSTA = 0x90;       // continuous RX
    BAUDCON = 0x08;     // BRG16 = 1
    temp = RCREG;       // Empty buffer
	temp = RCREG;       // Empty buffer
}
  
char ReceiveSerial(unsigned char *data)
{
	if (PIR1bits.RCIF ==0 ) return(0);
	*data = RCREG;
	return (1); 
}

void TXbyte(unsigned char data)
{
    while(TXSTAbits.TRMT==0);
    TXREG = data;
}


void SendString(unsigned char *ch)
{
	while (*ch != '\0')
	{
		TXbyte(*ch++);
	}
}


