//--------------------------------------------------------------------------------
// Example program how to use the Usb-API for the PIC 18F4550 Demo board
// Author: Gerhard Burger
// E-Mail: gCoolfire@yahoo.de
// Web: http://members.aon.at/geburger/
// Last Update: 4th of June 2005
// Version 1.0
// Compiler: Borland Delphi, Version 7.0
// This software is free for non commercial use, for commercial use, please contact
// the author
// Please read the Disclaimer
//---------------------------------------------------------------------------------
unit PICUSB;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, usbAPI, ExtCtrls, shellapi;

Const vid_pid  : PCHAR  = 'vid_04d8&pid_000c' + #0;    // Default Demo Application Firmware
      out_pipe : PCHAR = '\MCHP_EP1' + #0;
      in_pipe  : PCHAR = '\MCHP_EP1' + #0;

      READ_VERSION      = $00;
      WRITE_READ_DATA   = $04;
      SET_LED           = $06;

      UsbBufSize = 64;

Type  PByteBuffer = ^TByteBuffer;
      TByteBuffer = Array[0..63] of Byte;


// Example Structure for Communication between PC and PIC
      PUsbData = ^TUsbData;
      TUSBData = Record
                      Cmd:Byte;          // the command to execute
                      DataLen : Byte;    // length of Data
                      Data : Array[0..UsbBufSize-1] of Byte;  // data
                    End;

  TfrmUsbTest = class(TForm)
    Memo1: TMemo;
    btnGetUSBDriverVersion: TButton;
    btnClose: TButton;
    btnWriteReadData: TButton;
    btnGetSummary: TButton;
    btnReadUSBFirmwareVersion: TButton;
    Label1: TLabel;
    Panel1: TPanel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    chkSwitchLed3: TCheckBox;
    lblWebInfo: TLabel;
    Label8: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnWriteReadDataClick(Sender: TObject);
    procedure btnGetUSBDriverVersionClick(Sender: TObject);
    procedure btnGetSummaryClick(Sender: TObject);
    procedure btnReadUSBFirmwareVersionClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure chkSwitchLed3Click(Sender: TObject);
    procedure lblWebInfoClick(Sender: TObject);
  private
    { Private declarations }
  public
    myOutPipe : THANDLE;
    myInPipe  : THANDLE;
    SaveCol:TColor;
    Function SendReceivePacket(SendData :PByteBuffer; SendLength : DWORD ; ReceiveData : PByteBuffer;
                   var ReceiveLength : DWORD; SendDelay :Word; ReceiveDelay:Word):DWORD;
    Procedure CheckInvalidHandle();
    Function GetSummary():Integer;
    Procedure GetUSBDemoFWVersion();
    Function SetLed(State:Boolean):Integer;
  end;

var
  frmUsbTest: TfrmUsbTest;

implementation

{$R *.dfm}

procedure TfrmUsbTest.FormCreate(Sender: TObject);
begin
  myInPipe  := INVALID_HANDLE_VALUE;
  myOutPipe := INVALID_HANDLE_VALUE;
end;

// Note:
// The total number of devices using the generic driver could be
// bigger than max_count. They could have different vid & pid numbers.
// This means if max_count is 2, the valid instance index do not
// necessary have to be '0' and '1'.
//
// Below is a sample code for searching for all valid instance indexes.
// MAX_NUM_MPUSB_DEV is defined in _mpusbapi.h
Function TfrmUsbTest.GetSummary():Integer;
Var tempPipe:THandle;
    count:DWORD;
    max_Count:DWORD;
    i:Byte;
Begin
  tempPipe := INVALID_HANDLE_VALUE;
  count:=0;
  max_count := MPUSBGetDeviceCount(vid_pid);
	if(max_count=0) then
  Begin
		result:= max_count;
    Memo1.Lines.add('No device found');
    exit;
  End
	Else
    memo1.lines.add(IntToStr(max_Count) + ' device(s) with ' + vid_pid  +  ' currently attached');
  count := 0;
  For i:=0 to MAX_NUM_MPUSB_DEV-1 Do
  Begin
    tempPipe := MPUSBOpen(i,vid_pid,NIL,MP_READ,0);
    if(tempPipe <> INVALID_HANDLE_VALUE) then
    Begin
      memo1.lines.add('Instance Index ' + IntToStr(i));
      MPUSBClose(tempPipe);
      Inc(count);
    End;
    if(count = max_count) Then break;
  End;
  result:= max_count;
End;//end GetSummary

// The default demo firmware application has a defined application level protocol.
// To read the firmware version, the host must send the READ_VERSION
// command which is defined as 0x00, follows by the length of the
// expected result, in this case is 2 bytes, major version, and minor version.
// i.e. <READ_VERSION><0x02>
//
// The response packet from the board has the following format:
// <READ_VERSION><0x02><MINOR_VER><MAJOR_VER>
// The receive buffer size must be equal to or larger than the maximum
// endpoint size it is communicating with. In this case, 64 bytes.
Procedure TfrmUsbTest.GetUSBDemoFWVersion();
Var Selection : DWORD;
    RecvLength : DWORD;
    send_buf : TByteBuffer;
    receive_buf :TByteBuffer;
Begin
    Selection:=0; // according to our firmware, only index 0 is available!
    myOutPipe := MPUSBOpen(selection,vid_pid, out_pipe, MP_WRITE, 0);
    myInPipe  := MPUSBOpen(selection,vid_pid, in_pipe,  MP_READ,  0);
    If (myOutPipe = INVALID_HANDLE_VALUE) or (myInPipe = INVALID_HANDLE_VALUE) then
    Begin
      memo1.lines.add('Failed to open data pipes.');
      Exit;
    End;
    send_buf[0] := READ_VERSION; // Command for Read Firmware version
    send_buf[1] := $02;         // Expected length of the DATA result (Except 2 bytes header). change it as needed,here 0x02 indicate firmware return 2 bytes data.
  	RecvLength := 2 + send_buf[1]; // Command + Datalen + Minor version + Major version= 4 Byte
    if (SendReceivePacket(@send_buf,send_buf[1],@receive_buf,RecvLength,1000,1000) = 1) Then
    Begin
  		if (receive_buf[0] = READ_VERSION) then
        Memo1.lines.add('USB Demo Firmware Version ' + inttoStr(receive_buf[3]) + '.' + IntToStr(receive_buf[2]))
  		Else
        Memo1.lines.add('upper cerita is error!');
    End
    Else
      Memo1.lines.add('USB Operation Failed');
    MPUSBClose(myOutPipe);
    MPUSBClose(myInPipe);
    myOutPipe := INVALID_HANDLE_VALUE;
    myInPipe := INVALID_HANDLE_VALUE;
End;



// example procedure. The procedure shows how to write a package of data to the pic
// and read data back to the pic
// 48 byte of Data will be initialized with 0, 1, 2, 3, ... 47;
// These data will be sent to the PIC
// The PIC firmware reads the data, adds 1 to each value and send the modified data
// package back to the PC
// for easier Handling the data structure TUsbData is used (see on top)
procedure TfrmUsbTest.btnWriteReadDataClick(Sender: TObject);
var selection:DWORD;
    send_buf:TUsbData;
    receive_buf:TUsbData;
    RecvLength:DWORD;
    i:word;
    s:String;
Begin
  Selection:=0;
  myOutPipe := MPUSBOpen(selection, vid_pid, out_pipe, MP_WRITE, 0);
  myInPipe  := MPUSBOpen(selection, vid_pid, in_pipe , MP_READ,  0);
  if (myOutPipe = INVALID_HANDLE_VALUE) or (myInPipe = INVALID_HANDLE_VALUE) then
  Begin
    if (myOutPipe = INVALID_HANDLE_VALUE)  then  memo1.lines.add('Failed to open out data pipe');
    if (myInPipe  = INVALID_HANDLE_VALUE)  then  memo1.lines.add('Failed to open in data pipe');
    Exit;
  End;
  send_buf.Cmd:= WRITE_READ_DATA;
  send_buf.DataLen:= $30;
	for i := 0 to  send_buf.DataLen-1 Do
		send_buf.Data[i] := i;
	RecvLength:=2 + send_buf.DataLen; // 2 is header length in byte (1+1+3)
  if(SendReceivePacket(@send_buf,send_buf.DataLen + 2,@receive_buf,RecvLength,1000,1000) = 1) then
  Begin
		if(receive_buf.Cmd = WRITE_READ_DATA) then
    begin
			memo1.Lines.Add('Read '+ IntToStr(receive_buf.datalen) + ' bytes from Pic ');
      s:='';
     	For i:=0 to receive_buf.Datalen-1 do
			Begin
				If (i mod 10 = 0) then
        Begin
          memo1.Lines.add(s);
          s:='';
        End;
        If receive_buf.Data[i] < 10 then s:= s + '  0' + IntToStr(receive_buf.Data[i])
                                    else s:= s + '  '  + IntToStr(receive_buf.Data[i]);
      end;
      memo1.Lines.add(s); // add last line
    End
		Else
		Begin
      memo1.lines.add('upper cerita is error!');
		End; // end if(receive_buf[0] == CurrentAction)
    End // end if(SendReceivePacket(....))
    Else
      memo1.lines.add('USB Operation Failed');
    // Let's close the data pipes since we have nothing left to do..
    MPUSBClose(myOutPipe);
    MPUSBClose(myInPipe);
    myOutPipe := INVALID_HANDLE_VALUE;
    myInPipe := INVALID_HANDLE_VALUE;
end;


procedure TfrmUsbTest.btnGetUSBDriverVersionClick(Sender: TObject);
var temp:DWORD;
    versionInfo : Array[0..3] of Byte;
begin
  temp := MPUSBGetDLLVersion();
  move(temp,VersionInfo,sizeof(VersionInfo));
  memo1.text:='USB Driver Version ' + intTOStr(VersionInfo[0]) +  '.' + intTOStr(VersionInfo[1]) +  '.' + intTOStr(VersionInfo[2]) +  '.'+ intTOStr(VersionInfo[3]);
end;

Procedure TfrmUsbTest.CheckInvalidHandle();
Begin
  if (GetLastError() = ERROR_INVALID_HANDLE) then
  Begin
    MPUSBClose(myOutPipe);     // Most likely cause of the error is the board was disconnected.
    MPUSBClose(myInPipe);
    myOutPipe := INVALID_HANDLE_VALUE;
    myInPipe  := INVALID_HANDLE_VALUE;
  End
  else
    Memo1.lines.Add('Error Code ' + IntToStr(GetLastError()));
End; //end CheckInvalidHandle


Function TfrmUsbTest.SendReceivePacket(SendData :PByteBuffer; SendLength : DWORD ; ReceiveData : PByteBuffer;
          var ReceiveLength : DWORD; SendDelay :Word; ReceiveDelay:Word):DWORD;
var
   SentDataLength:  DWORD ;
   ExpectedReceiveLength: DWORD;
Begin
  ExpectedReceiveLength := ReceiveLength;
  if(myOutPipe <> INVALID_HANDLE_VALUE) and ( myInPipe <> INVALID_HANDLE_VALUE) then
  Begin
    if MPUSBWrite(myOutPipe,SendData,SendLength,@SentDataLength,SendDelay) <> 0 then
    Begin
      if(MPUSBRead(myInPipe,ReceiveData, ExpectedReceiveLength,@ReceiveLength,ReceiveDelay)) <> 0 then
      Begin
        if (ReceiveLength = ExpectedReceiveLength) Then
        Begin
          result:=1;   // Success!
          Exit;
        End
        else
        Begin
          if (ReceiveLength < ExpectedReceiveLength) then
          begin
            result:=2; // Partially failed, incorrect receive length
          End;
        End
      End
      else
          CheckInvalidHandle();
        End
        else
            CheckInvalidHandle();
    End;
  result:=0;  // Operation Failed
End;//end SendReceivePacket

procedure TfrmUsbTest.btnGetSummaryClick(Sender: TObject);
begin
   GetSummary();
end;

procedure TfrmUsbTest.btnReadUSBFirmwareVersionClick(Sender: TObject);
begin
  GetUSBDemoFWVersion();
end;

procedure TfrmUsbTest.btnCloseClick(Sender: TObject);
begin
  Close();
end;

procedure TfrmUsbTest.chkSwitchLed3Click(Sender: TObject);
Begin
  SetLed(chkSwitchLed3.Checked);
End;

Function TfrmUsbTest.SetLed(State:Boolean):Integer;
var selection:DWORD;
    send_buf:TUsbData;
    SentDataLength:DWORD;
Begin
  Selection:=0;
  myOutPipe := MPUSBOpen(selection, vid_pid, out_pipe, MP_WRITE, 0);
  if (myOutPipe = INVALID_HANDLE_VALUE) then
  Begin
    if (myOutPipe = INVALID_HANDLE_VALUE)  then result:=1;// memo1.lines.add('Failed to open out data pipe');
    Exit; //USB Operation Failed
  End;
  send_buf.Cmd := SET_LED;
  if State = True then send_buf.Data[0]:=1
                  else send_buf.Data[0]:=0;
  send_buf.DataLen:=3;
  if MPUSBWrite(myOutPipe,@Send_buf,Send_buf.DataLen,@SentDataLength,1000) <> 0 then
  Begin
    if (SentDataLength <> Send_buf.DataLen) Then
      memo1.lines.Add('Failure on sending command LED ON/OFF');
  End
  else
    CheckInvalidHandle();
  MPUSBClose(myOutPipe);
  myOutPipe := INVALID_HANDLE_VALUE;
  Result:=0;
end;


procedure TfrmUsbTest.lblWebInfoClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open','http://members.aon.at/geburger/index_en.htm',nil,nil, SW_SHOWNORMAL);
end;


end.

